@@ -13,8 +13,8 @@
 #    under the License.
 
 from django.conf import settings
-from django.core.urlresolvers import reverse
 from django.template import defaultfilters as d_filters
+from django.urls import reverse
 from django.utils.translation import pgettext_lazy
 from django.utils.translation import ugettext_lazy as _
 from django.utils.translation import ungettext_lazy
@@ -87,11 +87,12 @@ class DownloadBackup(tables.LinkAction):
                                'object_path': object_path})
 
     def allowed(self, request, datum):
-        legacy_swift_panel_enabled = True
+        legacy_swift_panel_enabled = False
         if ('swift_panel' in settings.HORIZON_CONFIG and
-                settings.HORIZON_CONFIG['swift_panel'] == 'angular'):
-            legacy_swift_panel_enabled = False
-        return legacy_swift_panel_enabled and datum.status == 'COMPLETED'
+                settings.HORIZON_CONFIG['swift_panel'] == 'legacy' and
+                datum.status == 'COMPLETED'):
+            legacy_swift_panel_enabled = True
+        return legacy_swift_panel_enabled
 
 
 class DeleteBackup(tables.DeleteAction):
