@@ -17,14 +17,12 @@
 from django.utils.translation import ugettext_lazy as _
 
 import horizon
-from openstack_dashboard.dashboards.project import dashboard
 
 
 class Clusters(horizon.Panel):
     name = _("Clusters")
     slug = 'database_clusters'
-    permissions = ('openstack.services.database',
-                   'openstack.services.object-store',)
+    permissions = ('openstack.services.database',)
 
 
-dashboard.Project.register(Clusters)
+# dashboard.Project.register(Clusters)
