@@ -12,11 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import binascii
-import logging
-
 from django.conf import settings
-from django.core.urlresolvers import reverse
+from django.urls import reverse
 from django.utils.translation import ugettext_lazy as _
 
 from horizon import exceptions
@@ -28,9 +25,11 @@ from openstack_dashboard.dashboards.project.instances \
     import utils as instance_utils
 from openstack_dashboard.dashboards.project.instances.workflows \
     import create_instance as dash_create_instance
+from oslo_log import log as logging
 
 
 from trove_dashboard import api
+from trove_dashboard.utils import common as common_utils
 
 LOG = logging.getLogger(__name__)
 
@@ -64,6 +63,11 @@ class SetInstanceDetailsAction(workflows.Action):
         }))
 
     def __init__(self, request, *args, **kwargs):
+        if args:
+            self.backup_id = args[0].get('backup', None)
+        else:
+            self.backup_id = None
+
         super(SetInstanceDetailsAction, self).__init__(request,
                                                        *args,
                                                        **kwargs)
@@ -92,7 +96,7 @@ class SetInstanceDetailsAction(workflows.Action):
             self._errors["datastore"] = self.error_class([msg])
         else:
             datastore, datastore_version = parse_datastore_and_version_text(
-                binascii.unhexlify(datastore_and_version))
+                common_utils.unhexlify(datastore_and_version))
             field_name = self._build_flavor_field_name(datastore,
                                                        datastore_version)
             flavor = self.data.get(field_name, None)
@@ -109,7 +113,7 @@ class SetInstanceDetailsAction(workflows.Action):
         datastore_and_version = context["datastore"]
         if datastore_and_version:
             datastore, datastore_version = parse_datastore_and_version_text(
-                binascii.unhexlify(context["datastore"]))
+                common_utils.unhexlify(context["datastore"]))
             field_name = self._build_flavor_field_name(datastore,
                                                        datastore_version)
             flavor = self.data[field_name]
@@ -184,18 +188,39 @@ class SetInstanceDetailsAction(workflows.Action):
             LOG.exception("Exception while obtaining datastore version list")
             self._datastore_versions = []
 
+    @memoized.memoized_method
+    def get_backup(self, request, backup_id):
+        try:
+            return api.trove.backup_get(request, backup_id)
+        except Exception:
+            LOG.exception("Exception while obtaining backup information")
+            return None
+
     def populate_datastore_choices(self, request, context):
         choices = ()
         datastores = self.datastores(request)
         if datastores is not None:
+            if self.backup_id:
+                backup = self.get_backup(request, self.backup_id)
             for ds in datastores:
+                if self.backup_id:
+                    if ds.name != backup.datastore['type']:
+                        continue
                 versions = self.datastore_versions(request, ds.name)
                 if versions:
                     # only add to choices if datastore has at least one version
                     version_choices = ()
                     for v in versions:
-                        if hasattr(v, 'active') and not v.active:
+                        # NOTE(zhaochao): please refer to the comment about
+                        # the same change for 'populate_datastore_choices'
+                        # of 'LaunchForm' in
+                        # trove_dashboard/content/database_clusters/forms.py
+                        # for details.
+                        if not v.to_dict().get('active', True):
                             continue
+                        if self.backup_id:
+                            if v.id != backup.datastore['version_id']:
+                                continue
                         selection_text = self._build_datastore_display_text(
                             ds.name, v.name)
                         widget_text = self._build_widget_field_name(
@@ -239,7 +264,7 @@ class SetInstanceDetailsAction(workflows.Action):
         # Since the fieldnames cannot contain an uppercase character
         # we generate a hex encoded string representation of the
         # datastore and version as the fieldname
-        return binascii.hexlify(
+        return common_utils.hexlify(
             self._build_datastore_display_text(datastore, datastore_version))
 
     def _build_flavor_field_name(self, datastore, datastore_version):
@@ -350,6 +375,18 @@ class AdvancedAction(workflows.Action):
             'data-initial_state-master': _('Replica Count')
         }))
 
+    def __init__(self, request, *args, **kwargs):
+        if args[0]:
+            self.backup_id = args[0].get('backup', None)
+        else:
+            self.backup_id = None
+
+        super(AdvancedAction, self).__init__(request, *args, **kwargs)
+
+        if self.backup_id:
+            self.fields['initial_state'].choices = [('backup',
+                                                    _('Restore from Backup'))]
+
     class Meta(object):
         name = _("Advanced")
         help_text_template = "project/databases/_launch_advanced_help.html"
@@ -374,9 +411,13 @@ class AdvancedAction(workflows.Action):
 
     def populate_backup_choices(self, request, context):
         try:
+            choices = []
             backups = api.trove.backup_list(request)
-            choices = [(b.id, b.name) for b in backups
-                       if b.status == 'COMPLETED']
+            for b in backups:
+                if self.backup_id and b.id != self.backup_id:
+                    continue
+                if b.status == 'COMPLETED':
+                    choices.append((b.id, b.name))
         except Exception:
             choices = []
 
@@ -520,8 +561,7 @@ class LaunchInstance(workflows.Workflow):
     def _get_nics(self, context):
         netids = context.get('network_id', None)
         if netids:
-            return [{"net-id": netid, "v4-fixed-ip": ""}
-                    for netid in netids]
+            return [{"net-id": netid} for netid in netids]
         else:
             return None
 
@@ -544,19 +584,19 @@ class LaunchInstance(workflows.Workflow):
     def handle(self, request, context):
         try:
             datastore, datastore_version = parse_datastore_and_version_text(
-                binascii.unhexlify(self.context['datastore']))
+                common_utils.unhexlify(self.context['datastore']))
             avail_zone = context.get('availability_zone', None)
             LOG.info("Launching database instance with parameters "
                      "{name=%s, volume=%s, volume_type=%s, flavor=%s, "
                      "datastore=%s, datastore_version=%s, "
-                     "dbs=%s, users=%s, "
+                     "dbs=%s, "
                      "backups=%s, nics=%s, replica_of=%s replica_count=%s, "
                      "configuration=%s, locality=%s, "
                      "availability_zone=%s}",
                      context['name'], context['volume'],
                      self._get_volume_type(context), context['flavor'],
                      datastore, datastore_version,
-                     self._get_databases(context), self._get_users(context),
+                     self._get_databases(context),
                      self._get_backup(context), self._get_nics(context),
                      context.get('master'), context['replica_count'],
                      context.get('config'), self._get_locality(context),
